# 🚀 {{ algorithm.title }} Template

Thank you for hosting your algorithm on Grand-Challenge.org, we appreciate it!

## Content

This Algorithm Template contains a tailored example to help you run [your
algorithm on Grand-Challenge.org]({{ algorithm.url }}).

It contains the following:
* ️🦾 A set of example bash scripts (.sh) to _help_ you test your algorithm locally and export it as a container for upload.
* 🦿 An example _python implementation_ that templates reading the inputs and writing the outputs.

See the header of the `inference.py` for more information.

Please note that this is supplementary to the [documentation](https://grand-challenge.org/documentation/algorithms/).
If the documentation does not answer your question, feel free to reach out to us at
[support@grand-challenge.org](mailto:support@grandchallenge.org).

The examples in this template illustrate one approach, but they are not the only option. Any container image will do as long as it:

{% for interface in algorithm.algorithm_interfaces %}
{% if not loop.first %}or input/outputs as follows:{% endif %}
{% for input in interface.inputs %}
- reads from the `/input/{{ input.relative_path}}`
{% endfor %}
{% for output in interface.outputs %}
- outputs, correctly, to `/output/{{ output.relative_path }}`
{% endfor %}
{% endfor %}
---
Generated by [Grand-Challenge-Forge](https://github.com/DIAGNijmegen/rse-grand-challenge-forge) v{{ grand_challenge_forge_version }}
