{% if algorithm_input_sockets | has_image or algorithm_output_sockets | has_image or phase.evaluation_additional_inputs | has_image or phase.evaluation_additional_outputs | has_image -%}
SimpleITK
numpy
{%- endif %}
psutil
